/* ----------------------------  Example RFID I.code Test   -------------------------------- */
//    Library ---> icode_rfid.h                                                              //
//    Module  ---> SL015M V2.1                                                               //
//    CardType --> I.code                                                                    //
//    Arduino  --> Uno R3                                                                    //
//    Connection Type --> TTL-RS232                                                          //
//       SoftwareSerial Use (PIN8  -> Rx)<------------------------>Tx-SL025M                 //
//                          (PIN9  -> Tx)<------------------------>Rx-SL025M                 //
//                          (PIN10 -> check_card)<---------------->IO-SL025M                 //
//       HardwareSerial PIN0,1 use for monitor and test module                               //
/* ---------------------------------------------------------------------------------------- */

#include <SoftwareSerial.h>
#include <icode_rfid.h>

/* ------------------------  Icode RFID Library  ------------------------ */
//  function   IcodeRFID(Rx,Tx);  :: SoftwareSerial(Rx,Tx); define Rx and Tx

IcodeRFID IcodeRFID(2,3);

//  function .begin(baudrate);   for example it's define baudrate = 9600
//  SoftwareSerial is use for ----->   SL015M Icode RFID Reader
//  HarewareSerial is use for ----->   PC for monitor and test function
void setup()
{
  IcodeRFID.begin(9600);
  Serial.begin(9600);
  Serial.println("");
  Serial.println("Hello RFID READER");
}

// ------------- Example Function   read_Taginfo(void)   ------------------- //
//  Use to read UID , AFI , DSFID and TYPE of I.code Card
//  
//  Function From Library
//    IcodeRFID.readTaginfo(byte *tag);
//    return :  0x00  (if operation succeed)
//    variable : byte *tag
//               tag -> uid[i]  // for store UID
//               tag -> afi     // for store afi
//               tag -> dsfid   // for store dsfid
//               tag -> type    // for store type
// --------------------------------------------------------------------------//
void read_Taginfo(void){
  taginfo *tag;
  tag = (taginfo *) malloc(sizeof(taginfo));
  if (IcodeRFID.readTaginfo(tag)==0x00){
	Serial.println("Operation Success");
        Serial.print ("UID  IS : "); 
        for(int i = 0 ;i<8;i++){
          String c = String(tag->uid[i],HEX);
          Serial.print(" ");
          Serial.print(c);
        }
        Serial.println("");  
        Serial.print("AFI : ");  Serial.println(tag->afi,HEX);  
        Serial.print("DSFID : ");  Serial.println(tag->dsfid,HEX);          
        Serial.print ("TYPE IS :  "); 
        switch(tag->type){
          case 0x31:
            Serial.println("Tag it");  
          break;
          case 0x32:
            Serial.println("I.code SLI");  
          break;            
        }     
  }
  free(tag);
  tag = NULL;  
}

// ------------- Example Function   test_readDATA(byte startblock,byte number);   ------------------- //
//  Use to read data in startblock to block = number ( one block it's have 4 byte )
//    variable : startblock  // use for define first block to read data
//             : number      // use for define number of block for read
//    example  : test_readDATA(0x01,2);   // it's read data block from block 0x01 and 0x02  
//
//  Function From Library
//    IcodeRFID.readBlock(byte startblock,byte number,byte *storedata); 
//    return :  0x00  (if operation succeed)
//    variable : startblock  // use for define first block to read data
//             : number      // use for define number of block for read
//             : storedata[i] // for store data from block after read
// ---------------------------------------------------------------------------------------------------//
void test_readDATA(byte startblock,byte number){
        datablock *storedata;
        storedata = (datablock *) malloc(sizeof(datablock));
        if (IcodeRFID.readBlock(startblock,number,storedata) == OPERATION_OK){
           Serial.println("Read DATA Complete");     
           Serial.print ("DATA Block "); 
           Serial.print (startblock,DEC);  Serial.print (" To ");  Serial.print (startblock+number,DEC); 
           Serial.print (" : "); 
           for(int i = 0 ;i<number*4;i++){
              String c = String(storedata->data[i],HEX);
              Serial.print(" ");
              Serial.print(c);
          } 
        }
        free(storedata);
        storedata = NULL; 
}



// ----- Example Function test_writeblock(byte block,byte byte1,byte byte2,byte byte3,byte byte4); ---- //
//  Use to write data in block
//    variable : block                           // use for define block , where you want to write data
//             : byte1 , byte2 , byte3 , byte4   // use for define data in block(4 byte)
//    example  : test_writeblock(0x02,0xA1,0xA2,0xA3,0xA4); // for write data [ 0xA1 0xA2 0xA3 0xA4 ]-block 0x02 
//
//  Function From Library
//    IcodeRFID.writeBlockFromData(byte block,byte *DAT_to_Write);
//    return :  0x00  (if operation succeed)
//    variable : block         // use for define block , where you want to write data
//             : DAT_to_Write  // use for send Array of data to write in a block
// ----------------------------------------------------------------------------------------------------//
void test_writeblock(byte block,byte byte1,byte byte2,byte byte3,byte byte4){
     byte DAT_to_Write[4] = {byte1,byte2,byte3,byte4};
        if (IcodeRFID.writeBlockFromData(block,DAT_to_Write) == OPERATION_OK){
           Serial.println("Write Block Complete");     
           Serial.print ("DATA Block "); 
           Serial.print (block,DEC); 
           Serial.print (" : "); 
           for(int i = 0 ;i<4;i++){
              String c = String(DAT_to_Write[i],HEX);
              Serial.print(" ");
              Serial.print(c);
          } 
        }
}


// ------------------------- Example Function test_writeAFI(byte AFI); ------------------------------- //
//  Use to write AFI in I.code Card
//    variable : AFI                      // use for define AFI value for write
//    example  : test_writeAFI(0x3A);     // Write AFI value = 0x3A to card
//
//  Function From Library
//    IcodeRFID.writeAFI(byte AFI);
//    return :  0x00  (if operation succeed)
//    variable : AFI          // use for define AFI value for write
// ----------------------------------------------------------------------------------------------------//
void test_writeAFI(byte AFI){

        if (IcodeRFID.writeAFI(AFI) == OPERATION_OK){
           Serial.println("Write AFI Complete");     
        }
}

// ------------------------- Example Function test_writeDSFID(byte DSFID); ------------------------------- //
//  Use to write DSFID in I.code Card
//    variable : DSFID                      // use for define AFI value for write
//    example  : test_writeDSFID(0xAA);     // Write DSFID value = 0xAA to card
//
//  Function From Library
//    IcodeRFID.writeDSFID(byte AFI);
//    return :  0x00  (if operation succeed)
//    variable : DSFID          // use for define DSFID value for write
// ----------------------------------------------------------------------------------------------------//
void test_writeDSFID(byte DSFID){

        if (IcodeRFID.writeDSFID(DSFID) == OPERATION_OK){
           Serial.println("Write DSFID Complete");     
        }
}


// ------------------------- Other Function For Set LED on SL015M  ---------------------------------- //
//  Function From Library
//    IcodeRFID.LED(byte led);   
//    return :  0x00  (if operation succeed)
//    variable : led           // use for setup status of LED > 0x01  led is on
//                                                            > 0x00  led is off

//    IcodeRFID.WaitStatus(int found_pin);   
//    return :  1  > if SL015M is found card
//              0  > if SL015M is not found card
//    variable : found_pin        // use for define pin as input
// ----------------------------------------------------------------------------------------------------//


// ----------------------------------------- Function loop  ------------------------------------------------------- //
//  This Function is already for test SL015M and I.code Card
/*
    Step1 > connect SL015M to Arduino and use (PIN 8) as Rx ---  use (PIN 9) as Tx --- use (PIN 10) as check found pin 
    Step2 > open Serial Monitor for monitor via hardware serial
    Step3 > put card on SL015M and if correct , Green LED is on
    Step4 > monitor is show string "" Found Card !!!");
    Step5 > press keyboard for next operation
            press  't' for test read taginfo function
            press  'r' for test read datablock function
            press  'w' for test write datablock function
            press  'a' for test write AFI function            
            press  'd' for test write DSFID function
            press  'L' for test ON LED function
            press  'l' for test OFF LED function
*/  // ** this program it's not work if card it's not put on SL015M or this module is not found card
//  Thankyou for test my program
// ---------------------------------------------------------------------------------------------------------------//
void loop() 
{  char a;
   byte startblock;
   byte number;
  if (IcodeRFID.WaitStatus(A5)){  // found card == 1     
      Serial.println("");
      Serial.println("Found Card !!!");
      int sector = 0;
          while(digitalRead(A5)==0){      // wait for next operation
            if (Serial.available() > 0){
              a = Serial.read();
              Serial.println("");              
              switch(a){
                case 't':
                  Serial.println("To Test Read TagInfo");
                  read_Taginfo();
                break;
                
                case 'r':
                  Serial.println("To Test Read DATA");
                  startblock = 0; number = 3;
                  test_readDATA(startblock,number);
                break;

                case 'w':
                  Serial.println("To Test Write Block");
                  test_writeblock(2,0x01,0x02,0x03,0x04);
                break;
                
                case 'a':
                  Serial.println("To Test Write AFI");
                  test_writeAFI(0xAF); // set for sector 0
                break;
                
                case 'd':
                  Serial.println("To Test Write DSFID");
                  test_writeDSFID(0xDD); // set for sector 0
                break;          
                
                case 'L':
                  Serial.println("To Test LED ON");
                  IcodeRFID.LED(0x01);
                break;
                
                case 'l':
                  Serial.println("To Test LED OFF");
                  IcodeRFID.LED(0x00);
                break;
              }
            }
    }    
  }
}
