#ifdef __cplusplus
extern "C" 
{
#endif
  #include <inttypes.h>
  #include <avr/io.h>
  #include <avr/interrupt.h>
#ifdef __cplusplus
}
#endif
#if ARDUINO >= 100
#include "Arduino.h"
#else   
#include "WProgram.h"
#endif


#include "icode_rfid.h"
unsigned char flag_read=0,pv_flag_read=1;
byte HEADER,LEN,CMD,STATUS,UID[8],TYPE,CHKSUM,BOX,AFI,DSFID;

IcodeRFID::IcodeRFID(int rx,int tx):SoftwareSerial(rx,tx)
{

}

void IcodeRFID::begin(long baudrate)
{
	SoftwareSerial::begin(baudrate);
}

bool IcodeRFID::WaitStatus(int found_pin){
   pinMode(found_pin,INPUT);  
   int input = digitalRead(found_pin);
   
   if(input==0)
   {
     if(pv_flag_read)
		flag_read=1;
   }
   if(input==1)
   {
     pv_flag_read=1;
   }
   
   if (flag_read == 1){
		pv_flag_read=0;
		flag_read=0; 
		return(1);
   }else	
	  return(0);
}

byte IcodeRFID::checksum(byte (*values),int length){
  byte result = values[0];
  for (int i = 1; i < length; i++)
    result ^= values[i];
  return result;
}
byte IcodeRFID::readTaginfo(taginfo *tag){
    SoftwareSerial::write(0xBA);
    SoftwareSerial::write(0x02);
    SoftwareSerial::write(0x31);   
    SoftwareSerial::write(0x89);
    while(1){
      if (SoftwareSerial::read() == 0xBD) break;
    }
    
      while (SoftwareSerial::available()==0);    
         LEN = SoftwareSerial::read();
    
      while (SoftwareSerial::available()==0);   
         CMD = SoftwareSerial::read();
    
      while (SoftwareSerial::available()==0) ;   
         STATUS = SoftwareSerial::read();
       
      if (STATUS == 0x00) { // Select Success
        for (int i = 0; i < 8 ; i++) {      
          while (SoftwareSerial::available() == 0) ;
          tag->uid[i] = SoftwareSerial::read();
        }  
      
        while (SoftwareSerial::available() == 0) ;    
          tag->afi = SoftwareSerial::read();
       
        while (SoftwareSerial::available() == 0) ;    
          tag->dsfid = SoftwareSerial::read();
        while (SoftwareSerial::available() == 0) ;    
          tag->type = SoftwareSerial::read();	      
      }
      while (SoftwareSerial::available()==0) ;   
         CHKSUM = SoftwareSerial::read();
		 
	  return(STATUS);
}

byte IcodeRFID::readBlock(int startblock,int number,datablock *storedata){
  byte *frame;
  frame = (byte *) malloc(6 * sizeof(byte));
  frame[0] = SEND_HEADER;
  frame[1] = 4;
  frame[2] = 0x33;
  frame[3] = startblock;
  frame[4] = number;
  frame[5] = checksum(frame, 5);

  // Send frame to reader  
  SoftwareSerial::write(frame, 6);
  free(frame);
  frame = NULL;
  
      while (1){    
		while (SoftwareSerial::available()==0); 
			if(SoftwareSerial::read() == 0xBD) break;
      }     
      while (SoftwareSerial::available()==0);    
         LEN = SoftwareSerial::read();
    
      while (SoftwareSerial::available()==0);   
         CMD = SoftwareSerial::read();
    
      while (SoftwareSerial::available()==0) ;   
         STATUS = SoftwareSerial::read();
                
      if (STATUS == 0x00) { // Select Success
		for (int i = 0; i < LEN - 3; i++) {      
			while (SoftwareSerial::available() == 0) ;
			storedata->data[i] = SoftwareSerial::read();
		}  
      }
	  
      while (SoftwareSerial::available() == 0) ;    
          CHKSUM = SoftwareSerial::read();
       
      if (STATUS == 0x00) // operation ok
		return (0x00);
	  else
		return (0x01);
      //else  Serial.println("Read UNcomplete");   
      
}

byte IcodeRFID::writeBlockFromData(int block,byte *DATAIN){
  byte *frame;
  frame = (byte *) malloc(9 * sizeof(byte));
  frame[0] = SEND_HEADER;
  frame[1] = 7;
  frame[2] = 0x34;  // write block
  frame[3] = block;
  frame[4] = DATAIN[0];
  frame[5] = DATAIN[1];
  frame[6] = DATAIN[2];
  frame[7] = DATAIN[3];  
  frame[8] = checksum(frame, 8); 

  // Send frame to reader  
  SoftwareSerial::write(frame, 9);
  free(frame);
  frame = NULL;
      while (1){    
         while (SoftwareSerial::available()==0); 
         if(SoftwareSerial::read() == 0xBD) break;
      }      
      while (SoftwareSerial::available()==0);    
         LEN = SoftwareSerial::read();  
         
      while (SoftwareSerial::available()==0);   
         CMD = SoftwareSerial::read();  
         
      while (SoftwareSerial::available()==0) ;   
         STATUS = SoftwareSerial::read();
       
      if (STATUS == 0x00) { // Select Success
        for (int i = 0; i < 4 ; i++) {      
          while (SoftwareSerial::available() == 0) ;
          DATAIN[i] = SoftwareSerial::read();
        }  
      }                    
      while (SoftwareSerial::available() == 0) ;    
          CHKSUM = SoftwareSerial::read();       
		  
      if (STATUS == 0x00) return (0x00);
}
byte IcodeRFID::writeAFI(byte AFI){
  byte *frame;
  frame = (byte *) malloc(5 * sizeof(byte));
  frame[0] = SEND_HEADER;
  frame[1] = 3;   // Length 10
  frame[2] = 0x36;
  frame[3] = AFI;
  frame[4] = checksum(frame, 4);

  // Send frame to reader  
  SoftwareSerial::write(frame, 5);
  free(frame);
  frame = NULL;
        while (SoftwareSerial::available()==0);    
           HEADER = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0);    
           LEN = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0);   
           CMD = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0) ;   
           STATUS = SoftwareSerial::read(); 
		
        while (SoftwareSerial::available()==0) ;   
           AFI	 = SoftwareSerial::read(); 	
 
         while (SoftwareSerial::available()==0) ;     
           CHKSUM = SoftwareSerial::read();
		   
		return(STATUS);  
}
byte IcodeRFID::writeDSFID(byte DSFID){
  byte *frame;
  frame = (byte *) malloc(5 * sizeof(byte));
  frame[0] = SEND_HEADER;
  frame[1] = 3;   // Length 10
  frame[2] = 0x35;
  frame[3] = DSFID;
  frame[4] = checksum(frame, 4);

  // Send frame to reader  
  SoftwareSerial::write(frame, 5);
  free(frame);
  frame = NULL;
        while (SoftwareSerial::available()==0);    
           HEADER = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0);    
           LEN = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0);   
           CMD = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0) ;   
           STATUS = SoftwareSerial::read(); 
		
        while (SoftwareSerial::available()==0) ;   
           DSFID = SoftwareSerial::read(); 	
 
         while (SoftwareSerial::available()==0) ;     
           CHKSUM = SoftwareSerial::read();
		   
		return(STATUS);  
}

byte IcodeRFID::LED(byte led){
  byte *frame;
  frame = (byte *) malloc(5 * sizeof(byte));
  frame[0] = SEND_HEADER;
  frame[1] = 3;   // Length 10
  frame[2] = 0x40;
  frame[3] = led;
  frame[4] = checksum(frame, 4);

  // Send frame to reader  
  SoftwareSerial::write(frame, 5);
  free(frame);
  frame = NULL;
        while (SoftwareSerial::available()==0);    
           HEADER = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0);    
           LEN = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0);   
           CMD = SoftwareSerial::read();
    
        while (SoftwareSerial::available()==0) ;   
           STATUS = SoftwareSerial::read(); 
 
         while (SoftwareSerial::available()==0) ;     
           CHKSUM = SoftwareSerial::read();
		   
		return(STATUS);  
}









