#ifndef icode_rfid_h
#define icode_rfid_h

#include <Arduino.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <SoftwareSerial.h>
#include <String.h>

// Reader commands
#define SEND_HEADER 0xBA

// Read status
#define OPERATION_OK 0x00
#define NO_TAG 0x01
#define READ_FAIL 0x04
#define WRITE_FAIL 0x05

// Login results
#define LOGIN_SUCCEED 0x02
#define LOGIN_FAIL 0x03

// Stores tag data
struct taginfo {
  byte uid[8];  // UID with 4 bytes
  byte type;
  byte afi;
  byte dsfid;
  byte status;
};
struct datablock {
  byte data[150];  // block with 4 bytes
};

class IcodeRFID : public SoftwareSerial 
{
	public:
		IcodeRFID(int rx,int tx);
		bool WaitStatus(int found_pin);
		void begin(long baudrate);
		byte readTaginfo(taginfo *tag);
		byte writeAFI(byte AFI);
		byte writeDSFID(byte DSFID);
		byte readBlock(int startblock,int number,datablock *storedata);
		byte writeBlockFromData(int block,byte *DATAIN);
		byte LED(byte led);
    private:
		byte checksum(byte *value,int length);
};

#endif
